#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Univention Management Console
#  module: Open Xchange License Management
#
# Copyright (C) 2010-2017 Univention GmbH <http://www.univention.de/>
# by order of
#     Open-Xchange (http://www.open-xchange.com)
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

from univention.lib import Translation
from univention.management.console.modules import Base
from univention.management.console.log import MODULE
from univention.management.console.config import ucr
import notifier.popen
import notifier.threads
import os
import traceback

import oxldbcore

_ = Translation('univention-management-console-module-oxldb').translate

FN_LICENSE_BLACKLIST = '/usr/share/univention-management-console-module-oxldb/blacklisted_signatures'


class Instance(Base):
	OPTION_MAPPING = ( ( 'username', 'ox/ldbaccount/username', '' ),
					   ( 'currentLicensekey', 'ox/licensekey', '' ),
					   ( 'ldapBase', 'ldap/base', ''),
					   ( 'serverRole', 'server/role', 'domaincontroller_master'),
					   )

	def __init__( self ):
		Base.__init__( self )

	def load( self, request ):
		"""Retrieve current configuration status of the Open Xchange system

		options: {}

		return: { <all OX related UCR variables> }
		"""
		result = {}
		ucr.load()
		for option, var, default in Instance.OPTION_MAPPING:
			result[ option ] = ucr.get( var, default )

		username = ucr.get('ox/ldbaccount/username')
		password = ''
		pwd_file = ucr.get('ox/ldbaccount/passwordfile')
		result['passwordExistsInBackend'] = bool(pwd_file and os.path.exists(pwd_file))
		if result['passwordExistsInBackend']:
			try:
				password = open(pwd_file, 'r').read().strip('\n\r')
			except IOError:
				password = ''
		result['credentialsValid'] = False

		def _thread(request):
			MODULE.info( 'trying to testconnect to OX server with username=%s' % username )
			oxldb = oxldbcore.OXLDB()
			connect_result = oxldb.check_credentials(username=username, password=password)
			MODULE.info( 'connect_result %s' % (connect_result))
			result['credentialsValid'] = (connect_result is True)
			return result

		if username and password:
			thread = notifier.threads.Simple('licensekey_get', notifier.Callback(_thread, request), notifier.Callback(self._thread_finished, request))
			thread.run()
		else:
			self.finished( request.id, result )

	def _check_thread_error( self, thread, result, request ):
		"""Checks if the thread returned an exception. In that case in
		error response is send and the function returns True. Otherwise
		False is returned."""
		if not isinstance( result, BaseException ):
			return False

		msg = '%s\n%s: %s\n' % ( ''.join( traceback.format_tb( thread.exc_info[ 2 ] ) ), thread.exc_info[ 0 ].__name__, str( thread.exc_info[ 1 ] ) )
		MODULE.process( 'An internal error occurred: %s' % msg )
		self.finished( request.id, None, msg, False )
		return True


	def _thread_finished( self, thread, result, request ):
		if self._check_thread_error( thread, result, request ):
			return
		self.finished( request.id, result )


	def licensekey_get( self, request ):
		"""Check given credentials and return a list of tuples containing OX licensekey and corresponding ldap base.

		options:
			username: customer id
			password: password
			passwordExistsInBackend: this boolean flag indicates if the original password has been unchanged and is known in backend

		return { 'success' : (True|False), licensekey: <LICENSEKEY> }
		"""
		def _thread(request):
			result = {
				'success': False,
				'licensekeys': [],
				'message': _('Connection to OX repository server failed!'),
			}

			username = request.options.get( 'username' )
			password = request.options.get( 'password' )
			if request.options.get( 'passwordExistsInBackend' ):
				pwd_file = ucr.get( 'ox/ldbaccount/passwordfile' )
				if pwd_file and os.path.exists(pwd_file):
					try:
						password = open(pwd_file,'r').read().strip('\n\r')
					except IOError:
						password = ''

			MODULE.info('trying to connect to OX servers with username=%s' % username)
			oxldb = oxldbcore.OXLDB()
			check_result = oxldb.check_credentials(username=username, password=password)
			MODULE.info('check_result %r' % (check_result,))
			if check_result is True:
				oxldb.username = username
				oxldb.password = password
				result['success'] = True
				result['message'] = _('Credentials have been tested successfully.')
				result['licensekey'] = oxldb.licensekey
			else:
				result['message'] = check_result
			MODULE.info( 'result %s' % (result,))
			return result

		thread = notifier.threads.Simple( 'licensekey_get', notifier.Callback( _thread, request ),
										  notifier.Callback( self._thread_finished, request ) )
		thread.run()

	def save(self, request):
		"""Save given credentials, import UCS license and update repository entries

		options:
			username: customer id
			password: password
			passwordExistsInBackend: this boolean flag indicates if the original password has been unchanged and is known in backend
			licensekey: OX license key

		return { 'success' : (True|False), 'message': <string> }
		"""
		def _thread( request ):
			result = {
				'success': True,
				'message': '',
			}

			oxldb = oxldbcore.OXLDB()

			username = request.options.get('username')
			password = request.options.get('password')
			if request.options.get('passwordExistsInBackend'):
				pwd_file = ucr.get(oxldb.UCR_OX_ACCOUNT_PASSWDFN)
				if pwd_file and os.path.exists(pwd_file):
					try:
						password = open(pwd_file, 'r').read().strip('\n\r')
					except IOError:
						password = ''
			licensekey = request.options.get('licensekey')

			MODULE.info('trying to connect to OX repo with username=%s' % (username,))
			connect_result = oxldb.check_credentials(username=username, password=password)
			MODULE.info('connect_result %s' % (result,))
			if connect_result is True:
				oxldb.username = username
				oxldb.password = password

				# update credentials
				MODULE.info('setting username...')
				msg = oxldb.set_username(username)
				if not msg:
					MODULE.info('setting password...')
					msg = oxldb.set_password(password)
				MODULE.info('updating components...')
				oxldb.update_components()
				if msg:
					result['message'] += _('<li>Failed to set customer id or password: %s</li>') % (msg,)
					result['success'] = False
					MODULE.error('Failed to set username/password: %s' % (msg,))
				else:
					result['message'] += '<li>Customer id and password have been set successfully.</li>'
					oxldb.update_components()

				# update license key
				msg = oxldb.set_license_key(licensekey)
				MODULE.info('set_license_key(%s) = %s' % (licensekey, msg))
				if msg is not True:
					result['message'] += _('<li>Failed to set license key: %s</li>') % (msg,)
					result['success'] = False
				else:
					result['message'] += _('<li>License key has been set successfully.</li>')

				if result['success']:
					result['message'] = _('<p>The following steps have been completed:<ul>%s</ul></p>') % result['message']
				else:
					result['message'] = _('<p>While updating settings some steps failed:<ul>%s</ul></p>') % result['message']

			else:
				result['message'] += _('<li>Connection to OX repository server failed - please try again (later)</li>')
				result['success'] = False
			MODULE.info('result %s' % (result,))
			return result

		thread = notifier.threads.Simple('save', notifier.Callback(_thread, request),
										 notifier.Callback(self._thread_finished, request))
		thread.run()
