#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010-2017 Univention GmbH <http://www.univention.de/>
# by order of
#     Open-Xchange (http://www.open-xchange.com)
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

import univention.management.console as umc

import re
import traceback
import os
import stat
import requests
from univention.management.console.log import MODULE
import univention.config_registry

_ = umc.Translation('univention-management-console-module-oxldb').translate


class OXLDB(object):
	UCR_ALLOWED_CHARACTERS = '^[^#:@]+$'
	OX_ALLOWED_CHARACTERS_USERNAME = '^.+$'  # allow all characters but at least one is required
	RE_ALLOWED_DEBIAN_PKGNAMES = re.compile('^[a-z0-9][a-z0-9.+-]+$')
	FN_UPDATER_APTSOURCES_COMPONENT = '/etc/apt/sources.list.d/20_ucs-online-component.list'
	# URL to repo server for credential check without protocol
	URL_OX_CREDENTIAL_CHECK = '://software.open-xchange.com/OX6/OXSEforUCS/4.0/'
	UCR_OX_ACCOUNT_FORCE_PROTOCOL = 'ox/ldbaccount/force/protocol'
	UCR_OX_ACCOUNT_SSLVERIFICATION = 'ox/ldbaccount/sslverification'
	UCR_OX_ACCOUNT_USER = 'ox/ldbaccount/username'
	UCR_OX_ACCOUNT_PASSWDFN = 'ox/ldbaccount/passwordfile'
	FN_OX_ACCOUNT_PASSWORD = '/etc/ox-secrets/ldb-account.secret'
	UCR_OX_COMPONENTS_AVAILABLE = 'ox/components/available'
	UCR_OX_LICENSEKEY = 'ox/licensekey'
	COMPONENT_DISABLED = 'disabled'
	COMPONENT_AVAILABLE = 'available'
	COMPONENT_PERMISSION_DENIED = 'permission denied'
	COMPONENT_UNKNOWN = 'unknown'
	# OX Keys:
	# OX6-SE4UCS-MK-
	# OX6-SE4UCS-SUB-
	# OX6-SE-MK-
	# OX6-SE-SUB-
	# OX-APPS-SUB-
	# OX-DOC-SUB-
	RE_OX_KEYS = re.compile('(^OX6?-(SE(4UCS)?|APPS|DOC)(-.*?)?-(MK|SUB)-|^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}$)')

	def __init__(self, check_credentials=False):
		self.configRegistry = univention.config_registry.ConfigRegistry()
		self.configRegistry.load()
		self.suds = None
		self.suds_sessionId = None
		self.keys_all = []

		self.licensekey = self.configRegistry.get(self.UCR_OX_LICENSEKEY, '').strip()
		self.username = self.configRegistry.get(self.UCR_OX_ACCOUNT_USER, '').strip()
		self.password_fn = self.configRegistry.get(self.UCR_OX_ACCOUNT_PASSWDFN, self.FN_OX_ACCOUNT_PASSWORD).strip()
		self.credentials_valid = False
		if not self.configRegistry.get(self.UCR_OX_ACCOUNT_PASSWDFN, '').strip():
			try:
				univention.config_registry.handler_set([u'%s=%s' % (self.UCR_OX_ACCOUNT_PASSWDFN,
																	self.password_fn)])
			except:
				pass

		try:
			self.password = open(self.password_fn, 'r').read().strip('\n\r')
		except:
			MODULE.info('OXLDB: init: reading password from %r failed; guessing password' % (self.password_fn,))
			self.password = self.guess_password()

		if check_credentials:
			self.check_credentials()

	def guess_password(self):
		if not self.username:
			return ''

		components = self.configRegistry.get(self.UCR_OX_COMPONENTS_AVAILABLE, '').split(',')
		for component in components:
			if component:
				if self.configRegistry.get('repository/online/component/%s' % (component,), '').lower() in ('yes', 'true', 'enable', 'enabled'):
					if self.configRegistry.get('repository/online/component/%s/username' % (component,)) == self.username:
						passwd = self.configRegistry.get('repository/online/component/%s/password' % (component,))
						if passwd:
							return passwd
		return ''

	def is_account_complete(self):
		return self.username and self.password

	def is_account_valid(self):
		return self.credentials_valid

	def check_credentials(self, username=None, password=None):
		"""
		Check credentials against repository server of OX.
		The result is also stored in self.credentials_valid.
		@param username Username to be used to for auth check
		@param password Password to be used to for auth check
		@return returns True if the credentials are valid; otherwise
                        an error message is returned
		"""

		# use given credentials or as fallback the known ones
		user = self.username
		if username:
			user = username
		passwd = self.password
		if password:
			passwd = password

		# stop here if credentials are missing
		if not user or not passwd:
			MODULE.error('OXLDB: check_credentials: credentials missing')
			return _('missing credentials')

		protocol = self.configRegistry.get(self.UCR_OX_ACCOUNT_FORCE_PROTOCOL, 'http')
		MODULE.info('OXLDB: check_credentials: using protocol %r' % (protocol,))
		verification = self.configRegistry.is_true(self.UCR_OX_ACCOUNT_SSLVERIFICATION, False)
		MODULE.info('OXLDB: check_credentials: sslverification=%r' % (verification,))
		if protocol == 'http':
			verification = False

		proxies = {
			'http': self.configRegistry.get('proxy/http', ''),
			'https': self.configRegistry.get('proxy/http', ''),
		}

		self.credentials_valid = False

		url = '%s%s' % (protocol, self.URL_OX_CREDENTIAL_CHECK)
		MODULE.info('OXLDB: check_credentials: URL=%r  USER=%r  PROXIES=%r' % (url, user, proxies))
		try:
			req = requests.head(url, auth=(user, passwd), proxies=proxies, verify=verification)
		except requests.exceptions.ConnectionError, ex:
			MODULE.error('OXLDB: check_credentials: failed to verify credentials: %r' % (ex,))
			return _('Connection error: establishing a connection to the OX repository server failed. Please check the network connection and (if required) the proxy settings.')
		MODULE.info('OXLDB: check_credentials: status_code=%r' % (req.status_code,))

		result = None
		if req.status_code == 200:
			self.credentials_valid = True
			result = True
		elif req.status_code == 404:
			result = _('Service failure: the service for verification of the given credentials is currently unavailable')
		elif req.status_code == 401:
			result = _('Invalid credentials: the verification of the given credentials failed')
		elif req.status_code == 403:
			result = _('Access forbidden: the access has been denied by the OX repository server during verfication of the given credentials')
		elif req.status_code == 407:
			result = _('Missing or wrong credentials for proxy authentication: this system is configured to use a HTTP proxy for accessing the internet. The proxy denied the access due to missing or wrong credentials. Please check and update the UCR variable "proxy/http".')
		elif req.status_code > 400:
			result = _('Service failure: an unusual error with status code %d occurred. Please try again (later)') % (req.status_code,)

		return result

	def set_license_key(self, licensekey):
		"""
		Write specified licensekey to UCR variable
		return value:
			True     ==> success
			<string> ==> error message
		"""
		try:
			univention.config_registry.handler_set([u'%s=%s' % (self.UCR_OX_LICENSEKEY, licensekey)])
			self.licensekey = licensekey
		except Exception, e:
			MODULE.error('OXLDB: set_license_key: %s' % (str(e), ))
			return _('Setting license key failed - please try again (later)')

		return True

	def clear_credentials(self):
		msg = None
		try:
			univention.config_registry.handler_unset([u'%s' % self.UCR_OX_ACCOUNT_USER,
													  u'%s' % self.UCR_OX_ACCOUNT_PASSWDFN])
			self.username = None
			self.password = None
		except Exception, ex:
			MODULE.error('Cannot reset username and password: %r' % (ex,))
			msg = _('Cannot reset username and password.')
		return msg

	def set_username(self, username):
		msg = None
		try:
			univention.config_registry.handler_set([u'%s=%s' % (self.UCR_OX_ACCOUNT_USER, username)])
			self.username = username
		except Exception, ex:
			MODULE.error('Cannot write username to UCR variable: %r' % (ex,))
			msg = _('Cannot write username to UCR variable.')
		return msg

	def set_password(self, password):
		self.configRegistry.load()
		self.password_fn = self.configRegistry.get(self.UCR_OX_ACCOUNT_PASSWDFN, self.FN_OX_ACCOUNT_PASSWORD).strip()

		msg = None
		if os.path.exists(self.password_fn) and not os.path.isfile(self.password_fn):
			msg = _('Cannot write password into file %r') % (self.password_fn,)
			MODULE.error('OXLDB: %s exists and is no file' % (self.password_fn,))
		else:
			try:
				open(self.password_fn, 'w+').write(password)
				os.chmod(self.password_fn, stat.S_IRUSR)
				self.password = password
			except:
				MODULE.error('OXLDB: Cannot write password into file %r:\n%s' % (self.password_fn,
																				 traceback.format_exc().replace('%', '#')))
				msg = _('Cannot write password into file %s') % (self.password_fn,)

		return msg

	def update_components(self):
		setlist = []
		components = self.configRegistry.get(self.UCR_OX_COMPONENTS_AVAILABLE, '').split(',')
		for component in components:
			if component:
				setlist.append(u'%s=%s' % ('repository/online/component/%s/username' % (component, ), self.username))
				setlist.append(u'%s=%s' % ('repository/online/component/%s/password' % (component, ), self.password))
		if setlist:
			try:
				univention.config_registry.handler_set(setlist)
			except Exception, ex:
				MODULE.error('Cannot set repository UCR variables: %r\nSETLIST = %r' % (ex, setlist))

if __name__ == '__main__':
	x = OXLDB()
	print x.check_credentials()
